<?php
/*
Plugin Name: WooCommerce Pay for Payment
Plugin URI: https://wordpress.org/plugins/woocommerce-pay-for-payment/
Description: Setup individual charges for each payment method in WooCommerce.
Version: 2.0.6
Author: Karolína Vyskočilová
Author URI: https://kybernaut.cz
License: GPL-2.0+
License URI: http://www.gnu.org/licenses/gpl-2.0.txt
Text Domain: woocommerce-pay-for-payment
Domain Path: /languages
*/

/**
 * Check if WooCommerce is active.
 *
 * @return void
 */
add_action( 'plugins_loaded', 'pay4payment_plugin_init' );
function pay4payment_plugin_init() {

	if ( current_user_can( 'activate_plugins' ) && ! class_exists( 'woocommerce' ) ) {

		add_action( 'admin_init',    'pay4payment_deactivate' );
		add_action( 'admin_notices', 'pay4payment_admin_notice' );

	} else {

		require_once plugin_dir_path( __FILE__ ) . '/inc/class-pay4pay.php';
		Pay4Pay::instance();

		if ( is_admin() )
			require_once plugin_dir_path( __FILE__ ) . '/inc/class-pay4pay-admin.php';
		}
}

/**
 * Display an alert to inform the admin why the plugin didn't activate
 *
 * @return void
 */
function pay4payment_admin_notice() {

	$pay4payment_plugin = __( 'WooCommerce Pay for Payment', 'woocommerce-pay-for-payment' );
	$woocommerce_plugin = __( 'WooCommerce', 'woocommerce-pay-for-payment' );

	echo '<div class="error"><p>'
		. sprintf( __( '%1$s requires %2$s. Please activate %2$s before activation of %1$s. This plugin has been deactivated.', 'woocommerce-pay-for-payment' ), '<strong>' . esc_html( $pay4payment_plugin ) . '</strong>', '<strong>' . esc_html( $woocommerce_plugin ) . '</strong>' )
		. '</p></div>';

	if ( isset( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
}

/**
 * Deactivate the plugin.
 *
 * @return void
 */
function pay4payment_deactivate() {
	deactivate_plugins( plugin_basename( __FILE__ ) );
}
